from django.contrib.auth.hashers import BasePasswordHasher, mask_hash
from django.utils.translation import gettext_noop as _

from . import checks  # noqa


class PlaintextPasswordHasher(BasePasswordHasher):
    """
    Password hashing, without the hashing.

    Store passwords in plaintext.
    """

    algorithm = "plaintext"

    def salt(self) -> str:
        return ""

    def decode(self, encoded):
        return {
            "algorithm": self.algorithm,
            "hash": encoded,
            "salt": None,
        }

    def harden_runtime(self, password, encoded):
        pass

    def safe_summary(self, encoded):
        decoded = self.decode(encoded)
        return {
            _("algorithm"): decoded["algorithm"],
            _("hash"): mask_hash(decoded["hash"], show=3),
        }

    def encode(self, password: str, salt: str):
        return f"{self.algorithm}$${password}"

    def verify(self, password: str, encoded: str) -> bool:
        # Constant-time comparison? What's that?
        return encoded == self.encode(password, "")


__all__ = ["PlaintextPasswordHasher"]



import pytest
from django.contrib.auth.hashers import (
    check_password,
    get_hashers_by_algorithm,
    identify_hasher,
    make_password,
)
from hypothesis import given
from hypothesis.strategies import text

from plaintext_password import PlaintextPasswordHasher

PASSWORD = "password123"


@given(text())
def test_makes_plaintext_password(password):
    hashed_password = make_password(password, hasher="plaintext")
    assert hashed_password == f"plaintext$${password}"


@given(text())
def test_check_password(password):
    assert check_password(password, f"plaintext$${password}")


@pytest.mark.parametrize("password", {"$password$", "!password!"})
def test_check_password_with_unsafe_char(password):
    hashed_password = make_password(password, hasher="plaintext")
    assert check_password(password, hashed_password)


@given(text())
def test_end_to_end(password):
    hashed_password = make_password(password, hasher="plaintext")
    assert check_password(password, hashed_password)


def test_identify_hasher():
    assert isinstance(identify_hasher("plaintext$$password"), PlaintextPasswordHasher)


@pytest.mark.parametrize("hasher", get_hashers_by_algorithm().keys())
def test_make_password_performance(hasher, benchmark):
    benchmark(make_password, PASSWORD, hasher=hasher)


@pytest.mark.parametrize("hasher", get_hashers_by_algorithm().keys())
def test_check_password_performance(hasher, benchmark):
    encoded_password = make_password(PASSWORD, hasher=hasher)
    benchmark(check_password, PASSWORD, encoded_password)



from django.contrib.auth.hashers import get_hashers_by_algorithm
from django.core import checks


@checks.register(checks.Tags.security, deploy=True)
def check_for_plaintext_passwords(app_configs, **kwargs):
    if "plaintext" in get_hashers_by_algorithm():
        yield checks.Critical(
            "Plaintext passwords should not be used in production.",
            hint="Remove the hasher from PASSWORD_HASHERS.",
        )
